/**
 *  A3
 *
 *  By Vincent Petry - PVince81 at yahoo dot fr
 *
 *  ---------------------------------------------------------------------------
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
 */
a3 = {};

(function(){
var Constants = {
    FIELD_SIZE: 50,
    BOARD_WIDTH: 16,
    BOARD_HEIGHT: 10
}

Constants.CORNER_SIZE = Constants.FIELD_SIZE / 2;

var Game = function(container, config) {
    config = config || {};
    this.container = container;
    this.gameView = new a3.GameView(container, config.mode);
    this.containerPos = this.container.cumulativeOffset();
    this.setupEvents();
    this.engine = new a3.Engine(this.gameView);
    if ( Game.debug ) {
        window.game = this;
    }
}

Game.prototype = {
    container: null,
    cursorPos: null,

    setupEvents: function() {
        var self = this;
        Event.observe(window, "resize", this.windowResized.bind(this));
        this.container.observe("mouseup", this.mouseUp.bind(this));
        this.container.observe("mousedown", this.mouseDown.bind(this));
        this.container.observe("mousemove", this.mouseMove.bind(this));
        this.container.observe("begindrag", function(e) { e.stop();});
        this.container.observe("contextmenu", function(e) {
            e.stop();
            self.rightClick();
        });
    },

    windowResized: function() {
        this.containerPos = this.container.cumulativeOffset();
    },

    start: function() {
        this.engine.start();
    },

    isPaused: function() {
        return this.engine.isPaused();
    },

    setPaused: function(pause) {
        this.engine.setPaused(pause);
    },

    mouseMove : function(ev) {
        if ( !this.engine.canInteract() ) {
            return;
        }
        var cursorPos = this.cursorPos,
            x = parseInt(Math.floor(( Event.pointerX(ev) - this.containerPos[0] ) / Constants.FIELD_SIZE)),
            y = parseInt(Math.floor(( Event.pointerY(ev) - this.containerPos[1] ) / Constants.FIELD_SIZE));
        
        if ( this.cursorPos && this.cursorPos.down && ( x != cursorPos.x || y != cursorPos.y ) ) {
            this.cursorPos.dragStarted = true;
        }
        ev.preventDefault();
    },

    mouseDown : function(ev) {
        if ( this.engine.canInteract() && ev.isLeftClick() ) {
            // start dragging
            var cursorPos = this.cursorPos,
                x = parseInt(Math.floor(( Event.pointerX(ev) - this.containerPos[0] ) / Constants.FIELD_SIZE)),
                y = parseInt(Math.floor(( Event.pointerY(ev) - this.containerPos[1] ) / Constants.FIELD_SIZE));
/*
            if ( cursorPos && cursorPos.x == x && cursorPos.y == y ) {
                // reclicking deselects
                this.cursorPos = null;
                this.gameView.clearCursor();
            }                
            else */
            if ( this.engine.board.canMoveItem(x, y) ) {
                // move cursor if not contiguous
                if ( cursorPos && (( cursorPos.x == x && Math.abs(cursorPos.y - y) == 1 ) ||
                    ( cursorPos.y == y && Math.abs(cursorPos.x - x) == 1 )) ) {
                    // TODO: reverse this condition
                }
                else {
                    this.cursorPos = {
                        x: x,
                        y: y,
                        down: true
                    };
                    this.gameView.setCursor(x, y);
                }
            }
            else {
                this.cursorPos = null;
                this.gameView.clearCursor();
            }
        }
        ev.preventDefault();
        return false;
    },
    
    mouseUp : function(ev) {
        if ( this.engine.canInteract() && ev.isLeftClick() ) {
            var cursorPos = this.cursorPos,
                x = parseInt(Math.floor(( Event.pointerX(ev) - this.containerPos[0] ) / Constants.FIELD_SIZE)),
                y = parseInt(Math.floor(( Event.pointerY(ev) - this.containerPos[1] ) / Constants.FIELD_SIZE)),
                movement = null;

            // check if contiguous to the previously selected item
            if ( cursorPos && (( cursorPos.x == x && Math.abs(cursorPos.y - y) > 0 ) ||
                ( cursorPos.y == y && Math.abs(cursorPos.x - x) > 0 )) ) {
                var vx = x - cursorPos.x;
                var vy = y - cursorPos.y;
                if ( cursorPos.dragStarted ) {
                    // allow moving the mouse further than the next block,
                    // normalize direction
                    if ( vx != 0 ) {
                        vx = vx / Math.abs(vx);
                    }
                    else {
                        vy = vy / Math.abs(vy);
                    }
                }

                if ( Math.abs(vx) == 1 || Math.abs(vy) == 1 ) {
                    // allow only movement of 1
                    movement = {
                        x: cursorPos.x,
                        y: cursorPos.y,
                        vx: vx,
                        vy: vy
                    }
                }
            }

            if ( movement && this.engine.board.canMoveItem(movement.x, movement.y) &&
                    this.engine.board.canMoveItem(movement.x + movement.vx, movement.y + movement.vy) ) {
                this.engine.moveItem(cursorPos.x, cursorPos.y, movement.vx, movement.vy);
                this.cursorPos = null;
                this.gameView.clearCursor();
            }
            else if ( cursorPos ) {
                cursorPos.down = false;
            }
            ev.preventDefault();
        }
    },

    rightClick: function() {
        this.cursorPos = null;
        this.gameView.clearCursor();
    }
}

a3.Constants = Constants;
a3.Game = Game;

})();