/**
 *  A3
 *
 *  By Vincent Petry - PVince81 at yahoo dot fr
 *
 *  ---------------------------------------------------------------------------
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
 */

(function() {
    var Constants = a3.Constants;
    var TimeView = function(container) {
        var self,
            el = document.createElement('span');
        el.className = "timeGauge ";
        this.el = el;
        this.totalTime = 1;
        this.currentTime = 0;
        this.maxHeight = container.offsetHeight;
        container.appendChild(el);
    }

    TimeView.prototype = {
        setTotalTime: function(totalTime) {
            this.totalTime = totalTime;
        },

        setRemainingTime: function(remainingTime) {
            this.remainingTime = remainingTime;
            this.el.style.height = Math.floor( ( this.totalTime - this.remainingTime ) * this.maxHeight / this.totalTime ) + "px";
        }
    }

    var MessageView = function(container) {
        var el = document.createElement('span'),
            self;
        el.className = "message ";
        el.draggable = false;
        this.textNode = document.createTextNode("");
        el.appendChild(this.textNode);
        this.el = el;
        container.appendChild(el);

    }

    MessageView.prototype = {
        remove: function() {
            if ( this.el ) {
                this.el.parentNode.removeChild(this.el);
            }
        },

        setText: function(message, type) {
            this.el.className = "message " + type;
            this.el.removeChild(this.textNode);
            this.textNode = document.createTextNode(message);
            this.el.appendChild(this.textNode);
            this.type = type;
        },

        show: function(timeOut) {
            var self = this;
            window.setTimeout(function() {
                self.el.className += " appear";
            }, 0);
            if ( timeOut ) {
                window.setTimeout(this.hide.bind(this), timeOut);
            }
        },

        hide: function() {
            var self = this;
            this.el.className = "message " + this.type + " disappear";
            var realHide = function() {
                self.el.stopObserving("transitionend", realHide);
                self.el.stopObserving("oTransitionEnd", realHide);
                self.el.stopObserving("webkitTransitionEnd", realHide);
                self.el.className += " hidden";
            }
            this.el.observe("transitionend", realHide);
            this.el.observe("oTransitionEnd", realHide);
            this.el.observe("webkitTransitionEnd", realHide);
            //window.setTimeout(this.remove.bind(this), 1000);
        }
    }

    var FieldView = function(container, model, x, y) {
        this.model = model;

        var el = document.createElement('span');
        el.className = "field ";

        if ( model.marbleCount == 1 ) {
            el.className += "block";
        }
        else if ( model.marbleCount > 1 ) {
            el.className += "marble";
        }

        el.style.left = x * Constants.FIELD_SIZE + "px";
        el.style.top = y * Constants.FIELD_SIZE + "px";

        if ( model.chains ) {
            this.chainsEl = document.createElement('span');
            this.chainsEl.className = "chain";
            this.chainsEl.style.left = x * Constants.FIELD_SIZE + "px";
            this.chainsEl.style.top = y * Constants.FIELD_SIZE + "px";
            this._removeChainClosure = this.removeChain.bind(this);
        }

        el.draggable = false;
        this.el = el;
        container.appendChild(el);
        this._removeClosure = this.remove.bind(this);
    }

    FieldView.prototype = {
        remove: function() {
            if ( this.el && this.el.parentNode ) {
                this.el.parentNode.removeChild(this.el);
            }
        },

        removeChain: function() {
            if ( this.chainsEl && this.chainsEl.parentNode ) {
                this.chainsEl.parentNode.removeChild(this.chainsEl);
            }
        },

        destroyAnimation: function() {
            this.el.className += " destroy";

            this.el.observe("transitionend", this._removeClosure);
            this.el.observe("oTransitionEnd", this._removeClosure);
            this.el.observe("webkitTransitionEnd", this._removeClosure);
        },

        destroyChain: function() {
            this.chainsEl.className += " destroy";

            this.chainsEl.observe("transitionend", this._removeChainClosure);
            this.chainsEl.observe("oTransitionEnd", this._removeChainClosure);
            this.chainsEl.observe("webkitTransitionEnd", this._removeChainClosure);
        }
    }

    var ItemView = function(container, model, x, y, spawning) {
        var self = this;
        this.model = model;
        this.x = x;
        this.y = y;
        this.spawning = spawning;
        var el = document.createElement('span');
        el.style.left = x * Constants.FIELD_SIZE + "px";
        el.style.top = y * Constants.FIELD_SIZE + "px";
        this.baseClassName = "item " + model.type;
        el.className = this.baseClassName;
        if ( spawning ) {
            el.className += " spawn";
            window.setTimeout(function() {
                el.className = self.baseClassName;
            }, 0);
        }

        el.draggable = false;
        this.el = el;
        this.id = ItemView.currentId++;
        container.appendChild(el);
        this._removeClosure = this.remove.bind(this);
    }

    ItemView.currentId = 0;
    ItemView.prototype = {
        remove: function() {
            if ( this.el && this.el.parentNode ) {
                this.el.parentNode.removeChild(this.el);
            }
        },

        collect: function() {
            var self = this,
                el = this.el;
            el.className += " collect";
            /*
            this.el.observe("transitionend", this._removeClosure);
            this.el.observe("oTransitionEnd", this._removeClosure);
            this.el.observe("webkitTransitionEnd", this._removeClosure);
            */
            // FIXME: can't observe event because of hover animation
            // that will trigger the event once the mouse is out
            window.setTimeout(this._removeClosure, 200);
        },

        /**
        * Animate movement of this item to the given target board position.
        * @param x target x coordinate
        * @param y target y coordinate
        */
        moveTo: function(x, y) {
            this.el.style.left = x * Constants.FIELD_SIZE + "px";
            this.el.style.top = y * Constants.FIELD_SIZE + "px";
            this.x = x;
            this.y = y;
        },

        failedMoveTo: function(x, y) {
            var self = this;

            var direction = "up";
            if ( x > this.x ) {
                direction = "right";
            }
            else if ( x < this.x ) {
                direction = "left";
            }
            else if ( y > this.y ) {
                direction = "down";
            }

            this.el.className = this.baseClassName + " bounce-" + direction;

            var bounceEnds = function() {
                /*
                self.el.stopObserving("transitionend", bounceEnds);
                self.el.stopObserving("oTransitionEnd", bounceEnds);
                self.el.stopObserving("webkitTransitionEnd", bounceEnds);
                */
                self.el.className = self.baseClassName;
            };

            // FIXME: can't observe event because of hover animation
            // that will trigger the event once the mouse is out
            window.setTimeout(bounceEnds, 500);
            /*
            this.el.observe("transitionend", bounceEnds);
            this.el.observe("oTransitionEnd", bounceEnds);
            this.el.observe("webkitTransitionEnd", bounceEnds);
            */
        }
    }

    var CornerView = function(container, type, x, y) {
        var el = document.createElement('span');
        el.className = "corner " + type;
        el.style.left = x * Constants.CORNER_SIZE + "px";
        el.style.top = y * Constants.CORNER_SIZE + "px";

        el.draggable = false;
        this.el = el;
        container.appendChild(el);
        this._removeClosure = this.remove.bind(this);
    }

    CornerView.prototype = {
        remove: function() {
            if ( this.el && this.el.parentNode ) {
                this.el.parentNode.removeChild(this.el);
            }
        }
    }

    var BoardView = function(container) {
        this.el = document.createElement('div');
        this.el.className = "board disappear";
        this.el.style.width = Constants.FIELD_SIZE * Constants.BOARD_WIDTH + "px";
        this.el.style.height = Constants.FIELD_SIZE * Constants.BOARD_HEIGHT + "px";

        this.board = null;
        this.fieldViews = null;
        this.itemViews = null;

        this.fieldsContainer = document.createElement('div');
        this.fieldsContainer.className = "fields";
        this.el.appendChild(this.fieldsContainer);

        this.itemsContainer = document.createElement('div');
        this.itemsContainer.className = "items";
        this.el.appendChild(this.itemsContainer);

        this.itemsOverlayContainer = document.createElement('div');
        this.itemsOverlayContainer.className = "itemsOverlay";
        this.el.appendChild(this.itemsOverlayContainer);

        this.cornersContainer = document.createElement('div');
        this.cornersContainer.className = "corners";
        this.el.appendChild(this.cornersContainer);

        container.appendChild(this.el);
    }

    BoardView.prototype = {

        show: function() {
            this.el.className = "board";
        },

        hide: function() {
            this.el.className = "board disappear";
        },

        _initFields : function() {
            var fields = this.board.fields;
            var views = [];

            for ( var y = 0; y < fields.length; y++ ) {
                var row = fields[y];
                var viewRow = [];
                for ( var x = 0; x < row.length; x++ ) {
                    var field = row[x];
                    if ( field && !field.generator ) {
                        var fieldView = new FieldView(this.fieldsContainer, field, x, y);
                        viewRow.push( fieldView );
                        if ( fieldView.chainsEl ) {
                            this.itemsOverlayContainer.appendChild(fieldView.chainsEl);
                        }
                    }
                    else {
                        viewRow.push(null);
                    }
                }
                views.push(viewRow);
            }
            return views;
        },

        _initItems : function() {
            var fields = this.board.items;
            var views = [];

            for ( var y = 0; y < fields.length; y++ ) {
                var row = fields[y];
                var viewRow = [];
                for ( var x = 0; x < row.length; x++ ) {
                    var field = row[x];
                    if ( field ) {
                        viewRow.push( new ItemView(this.itemsContainer, field, x, y) );
                    }
                    else {
                        viewRow.push(null);
                    }
                }
                views.push(viewRow);
            }
            return views;
        },

        _initCorners : function() {
            var fields = this.board.corners;
            var views = [];

            for ( var y = 0; y < fields.length; y++ ) {
                var row = fields[y];
                var viewRow = [];
                for ( var x = 0; x < row.length; x++ ) {
                    var field = row[x];
                    if ( field ) {
                        viewRow.push( new CornerView(this.cornersContainer, field, x, y) );
                    }
                    else {
                        viewRow.push(null);
                    }
                }
                views.push(viewRow);
            }
            return views;
        },

        _destroyViews : function(views) {
            if ( !views ) {
                return;
            }

            for ( var i = 0; i < views.length; i++ ) {
                for ( var j = 0; j < views[i].length; j++ ) {
                    if ( views[i][j] ) {
                        views[i][j].remove();
                    }
                }
            }
        },

        _destroyFields : function() {
            this._destroyViews(this.fieldViews);
            this.fieldViews = null;
        },

        _destroyItems : function() {
            this._destroyViews(this.itemViews);
            this.itemViews = null;
        },

        _destroyCorners : function() {
            this._destroyViews(this.cornerViews);
            this.cornerViews = null;
        },

        _clear: function() {
            if ( this.board ) {
                // TODO: maybe just destroy and recreate the layer?
                this._destroyFields();
                this._destroyItems();
                this._destroyCorners();
            }
        },

        setBoard: function(board) {
            this._clear();
            this.board = board;
            this.fieldViews = this._initFields();
            this.itemViews = this._initItems();
            this.cornerViews = this._initCorners();
            this.setImage(this.board.getImageName());
        },

        getFieldView: function(x, y) {
            return this.fieldViews[y][x];
        },

        setFieldView: function(x, y, view) {
            this.fieldViews[y][x] = view;
        },

        getItemView: function(x, y) {
            return this.itemViews[y][x];
        },

        setItemView: function(x, y, view) {
            this.itemViews[y][x] = view;
        },

        spawnItem: function(x, y, model) {
            var view = new ItemView(this.itemsContainer, model, x, y, true);
            this.setItemView(x, y, view);
        },

        destroyMarble: function(x, y, model) {
            var marbleView = this.getFieldView(x, y);
            if ( marbleView ) {
                marbleView.destroyAnimation();
            }
            var view = new FieldView(this.fieldsContainer, model, x, y);
            this.setFieldView(x, y, view);
        },

        destroyChain: function(x, y, model) {
            var marbleView = this.getFieldView(x, y);
            if ( marbleView ) {
                marbleView.destroyChain();
            }
        },

        setImage: function(imageName) {
            this.el.style.backgroundImage = "url(img/" + imageName + ")";
        },

        remove : function() {
            this._clear();
        }
    }

    var GameView = function(container) {
        this.container = container;
        this.boardView = new BoardView(container);

        loadCss("css/a3_elements.css");

        var el = document.createElement('span');
        el.className = "cursor hidden";
        el.draggable = false;
        this.cursorEl = el;
        this.boardView.el.appendChild(el);

        el = document.createElement('span');
        el.className = "overlay disappear";
        el.draggable = false;
        this.overlayEl = el;
        this.boardView.el.appendChild(el);

        // TODO: use internal element!
        this.scoreEl = document.getElementById("score");

        this.message = new MessageView(this.boardView.el);
        this.timeView = new TimeView(document.getElementsByClassName("timeGaugeContainer")[0]);
    }

    GameView.prototype = {

        showBoard: function(visible) {
            if ( visible ) {
                this.boardView.show();
            }
            else {
                this.boardView.hide();
            }
        },

        setBoard: function(board) {
            this.boardView.setBoard(board);
        },

        setCursor: function(x, y) {
            this.cursorEl.style.left = x * Constants.FIELD_SIZE + "px";
            this.cursorEl.style.top = y * Constants.FIELD_SIZE + "px";
            this.cursorEl.className = "cursor";
        },

        clearCursor: function() {
            this.cursorEl.className = "cursor hidden";
        },

        /**
        * Swap the items at the two given positions.
        */
        swapItems: function(x1, y1, x2, y2) {
            var firstItem = this.boardView.getItemView(x1, y1);
            var secondItem = this.boardView.getItemView(x2, y2);
            this.boardView.setItemView(x1, y1, secondItem);
            this.boardView.setItemView(x2, y2, firstItem);
            firstItem.moveTo(x2, y2);
            secondItem.moveTo(x1, y1);
        },

        /**
        * Try to swap the items at the two given positions,
        * but make it fail (for invalid moves).
        */
        failedSwapItems: function(x1, y1, x2, y2) {
            var firstItem = this.boardView.getItemView(x1, y1);
            var secondItem = this.boardView.getItemView(x2, y2);
            firstItem.failedMoveTo(x2, y2);
            secondItem.failedMoveTo(x1, y1);
        },

        moveItem: function(x1, y1, x2, y2) {
            var item = this.boardView.getItemView(x1, y1);
            if ( !item ) {
                return;
            }
            this.boardView.setItemView(x1, y1, null);
            this.boardView.setItemView(x2, y2, item);
            item.moveTo(x2, y2);
        },

        collectItem: function(x, y) {
            var item = this.boardView.getItemView(x, y);
            item.collect();
            this.boardView.setItemView(x, y, null);
        },

        spawnItem: function(x, y, model) {
            this.boardView.spawnItem(x, y, model);
        },

        destroyMarble: function(x, y, newModel) {
            this.boardView.destroyMarble(x, y, newModel);
        },

        destroyChain: function(x, y) {
            this.boardView.destroyChain(x, y);
        },

        showMessage: function(message, type, timeOut) {
            // TODO: message list ? cached message presets ?
            this.message.setText(message, type || "general");
            this.message.show(timeOut);
        },

        showOverlay: function(show) {
            if ( show ) {
                this.overlayEl.className = "overlay";
            }
            else {
                this.overlayEl.className = "overlay disappear";
            }
        },

        setScore: function(score) {
            this.scoreEl.innerHTML = score;
        },

        setTotalTime: function(totalTime) {
            this.timeView.setTotalTime(totalTime);
        },

        setRemainingTime: function(remainingTime) {
            this.timeView.setRemainingTime(remainingTime);
        }
    }

    a3.TimeView = TimeView;
    a3.MessageView = MessageView;
    a3.FieldView = FieldView;
    a3.ItemView = ItemView;
    a3.BoardView = BoardView;
    a3.GameView = GameView;

})();
