/**
 *  A3
 *
 *  By Vincent Petry - PVince81 at yahoo dot fr
 *
 *  ---------------------------------------------------------------------------
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
 */

(function(){
var Field = a3.Field;
var Item = a3.Item;

//var TEST_LEVEL = 6;
var TEST_LEVEL = null;
var levels = {
    1: {
        autoGenerators: true,
        imageName: "img1.jpg",
        totalTime: 100,
        build: function(board) {
            for ( var x = 4; x < board.width - 5; x++ )
            for ( var y = 2; y < board.height - 2; y++ ) {
                board.setField(x, y, new Field({marbleCount: 2}));
            }

            for ( var x = 4; x < board.width - 5; x++ ) {
                board.setField(x, 2, new Field({marbleCount: 1}));
                board.setField(x, board.height - 3, new Field({marbleCount: 1}));
            }

            for ( var y = 2; y < board.height - 2; y++ ) {
                board.setField(4, y, new Field({marbleCount: 1}));
                board.setField(board.width - 5, y, new Field({marbleCount: 1}));
            }
        }
    },

    2: {
        imageName: "img1.jpg",
        totalTime: 120,
        map: [
            "   GG       GG  ",
            "   OO       OO  ",
            "  OXXO     OXXO ",
            "  OXXOOGGGOOXXO ",
            "   OOOXXXXXOOO  ",
            "    OOXXXXXOO   ",
            "     OXXXXXO    ",
            "      XXXXX     ",
            "       XXX      ",
            "                ",
            "                "]
    },

     3: {
        totalTime: 300,
        map: [
            "    GGGGGGG     ",
            "    OOOXOOO     ",
            "    OOXXXOO     ",
            "    OX###XO     ",
            "    OX###XO     ",
            "    OX###XO     ",
            "    OOX XOO     ",
            "     OOXOO      ",
            "      OOO       ",
            "                "]
    },

     4: {
        totalTime: 400,
        map: [
            " XXXXXXXXXXXXXX ",
            " XXXXXXXXXXXXXX ",
            "  XXXXX  XXXXX  ",
            "  ####    ####  ",
            "   oo      oo   ",
            "   oo      oo   ",
            "  xxxx    xxxx  ",
            " xxxxxx  xxxxxx ",
            " xxxxxxxxxxxxxx ",
            " xxxxxxxxxxxxxx "]
    },
    
    5: {
        totalTime: 400,
        map: [
            "       OO       ",
            "       OO       ",
            "GGG  G OO G  GGG",
            "OOO  OOOOOO  OOO",
            "OXX  OOOOOO  XXO",
            "OXXXXXXXXXXXXXXO",
            "   OOXXXXXXOO   ",
            "   XXXX  XXXX   ",
            "  XXXX    XXXX  ",
            " XXXX      XXXX "]
    },
    
    6: {
        imageName: "img1.jpg",
        totalTime: 180,
        autoGenerators: true,
        map: [
        "XXXXXXXXXXXXXXXX",
        "XXXXXXXXXXXXXXXX",
        "XXXXXXXXXXXXXXXX",
        "XXXXX      XXXXX",
        "XXXXX      XXXXX",
        "XXXXX      XXXXX",
        "XXXXX      XXXXX",
        "XXXXXXXXXXXXXXXX",
        "XXXXXXXXXXXXXXXX",
        "XXXXXXXXXXXXXXXX"]
    },

    7: {
        imageName: "img1.jpg",
        totalTime: 180,
        map: [
            "XXXX        XXXX",
            "XXXX        XXXX",
            "XXXX        XXXX",
            "XXXX        XXXX",
            "    O       O   ",
            "     O GGG O    ",
            "      XXXXX     ",
            "      XXXXX     ",
            "      XXXXX     ",
            "      XXXXX     ",
            "      XXXXX     "]
    },

    "diagonalTest": {
        totalTime: 600,
        map: [
            "    G     G     ",
            "    O     O     ",
            "     O   O      ",
            "      O O       ",
            "       O        ",
            "GGGG       GGGGG",
            "oooo  ooo  ooooo",
            "oooo ooooo ooooo",
            "oooo ooooo ooooo",
            "Xooo ooooo ooooX"]
    },

    "diagonalTest2": {
        totalTime: 600,
        map: [
            "                ",
            "                ",
            "       G        ",
            "     OOOOO      ",
            "     OOoOO      ",
            "     OOoOO      ",
            "     OOoOO      ",
            "     OOOOO      ",
            "                ",
            "X              X"]
    },

     "test": {
        totalTime: 400,
        map: [
            " XXXXXXXXXXXXXX ",
            " XXXXXXXXXXXXXX ",
            "  XXXXX  XXXXX  ",
            "  ####    ####  ",
            "   oo      oo   ",
            "   oo      oo   ",
            "  xxxx    xxxx  ",
            " xxxxxx  xxxxxx ",
            " xxxxxxxxxxxxxx ",
            " xxxxxxxxxxxxxx "]
    }
};

var Levels = function() {
}

Levels.prototype = {
    getLevel: function(levelNum) {
        var levelSpec = levels[TEST_LEVEL || levelNum];
        return this._buildLevel(levelSpec);
    },

    _buildLevel: function(levelSpec) {
        var board = new a3.Board();

        if ( levelSpec.map ) {
            this._makeBoardFromString(board, levelSpec.map);
        }

        board.setImageName(levelSpec.imageName || "img1.jpg");
        board.setTotalTime(levelSpec.totalTime || 120);
        
        if ( levelSpec.build ) {
            levelSpec.build(board);
        }

        if ( levelSpec.autoGenerators ) {
            // generate generators automatically
            this._makeGenerators(board);
        }
        
        return board;
    },

    _makeBoardFromString: function(board, s) {
        var mapping = {
            " ": [null],
            "G": [{generator:true}],
            "O": [{marbleCount: 1}],
            "o": [{marbleCount: 1}, Item.TYPE_NOPOPULATE],
            "X": [{marbleCount: 2}],
            "x": [{marbleCount: 2}, Item.TYPE_NOPOPULATE],
            "#": [{marbleCount: 2, chains: 1}]
        };
        var EMPTY_FIELD = new Field();
        var maxY = (s.length > board.height)?board.height:s.length;
        for ( var y = 0; y < maxY; y++ ) {
            var maxX = (s[y].length > board.width)?board.width:s[y].length;
            for ( var x = 0; x < maxX; x++ ) {
                var m = mapping[s[y][x]] || [{}];
                if ( m[0] ) {
                    board.setField(x, y, new Field(m[0]));
                }
                else {
                    board.setField(x, y, EMPTY_FIELD);
                }
                if ( m[1] ) {
                    board.setItem(x, y, m[1]);
                }
            }
        }
    },

    _makeGenerators: function(board) {
        for ( var y = 1; y < board.height; y ++ )
        for ( var x = 0; x < board.width; x++ ) {
            if ( board.hasMarble(x, y) && !board.hasMarble(x, y - 1) ) {
                board.setField(x, y - 1, new Field({generator: true}));
            }
        }
    }
}

a3.Levels = Levels;

})();