/**
 *  Canvas sprites library
 *
 *  By Vincent Petry - PVince81 at yahoo dot fr
 *
 *  ---------------------------------------------------------------------------
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
 */
(function() {
    function createContext(canvas) {
        return canvas.getContext("2d");
    };

    var Renderer = function(canvas) {
        var context = createContext(canvas);
        if ( !context ) {
            throw "Canvas init failed";
        }
        this.context = context;
        this.context.viewportWidth = canvas.clientWidth;
        this.context.viewportHeight = canvas.clientHeight;
    };

    Renderer.prototype = {
        drawTexture : function(texture, x, y, width, height) {
            var context = this.context;
            context.drawImage(texture._image, x, y);
        },

        drawSprite : function(sprite, x, y, alpha, scale) {
            var context = this.context,
                width = sprite.width,
                height = sprite.height;
            if ( scale === undefined || scale === 1.0 ) {
                scale = 1.0;                
            }
            else {
                width *= scale;
                height *= scale;
                x -= (width - sprite.width) / 2;
                y -= (height - sprite.height) / 2;
            }

            if ( alpha === undefined ) {
                alpha = 1.0;
            }
            context.globalAlpha = alpha;
            context.drawImage(sprite.texture._image, sprite.x, sprite.y, sprite.width, sprite.height, x, y, width, height);
        },

        createTexture: function(imageSrc, callback) {
            return new Texture(imageSrc, callback);
        },

        createSprite: function(texture, x, y, width, height) {
            return new Sprite(texture, x, y, width, height);
        },

        clear: function() {
            this.context.clearRect(0, 0, this.context.viewportWidth, this.context.viewportHeight);
        }
    };

    var Texture = function(imageSrc, callback) {
        var self = this;
        this.loaded = false;
        this._image = new Image();
        this._image.onload = function() {
            self.loaded = true;
            if ( callback ) {
                callback(self);
            }
        }
        this._image.src = imageSrc;
    };

    Texture.prototype = {
    };

    var Sprite = function(texture, x, y, width, height) {
        this.texture = texture;
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
    }

    Sprite.prototype = {
    };

    window.CanvasRenderer = Renderer;

})();